'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var template = _interopDefault(require('@babel/template'));
var core = require('@babel/core');
var pug = require('pug');

const isNotEmptyLine = (ln) => (
  ln !== '' && !ln.match(/^\s*$/g)
);


const trimLeft = (pattern) => (ln) => ln.replace(
  new RegExp(`^${pattern}`),
  '',
);


const prepareRaw = (node) => {
  const { raw } = node.quasi.quasis[0].value;
  const lines = raw.split('\n').filter(isNotEmptyLine);
  const rootIndent = /^\s*/.exec(lines[0])[0];

  return lines.map(trimLeft(rootIndent)).join('\n');
};


const plugin = function babelPluginTransformPugJs() {
  return {
    visitor: {
      TaggedTemplateExpression(path) {
        if (path.node.tag.name === 'pug') {
          const prepared = prepareRaw(path.node);
          const compiled = pug.compileClient(
            prepared,
            { inlineRuntimeFunctions: false },
          );
          const transformed = core.transform(compiled);
          const parsed = template.ast(transformed.code);
          path.replaceWithMultiple(parsed);
        }
      },
    },
  };
};

module.exports = plugin;
